import { Component, inject, OnInit } from '@angular/core';
import { FormControl, FormGroup, ReactiveFormsModule, Validators } from '@angular/forms';
import { MatButtonModule } from '@angular/material/button';
import { MatCardModule } from '@angular/material/card';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIconModule } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { HermesClientService } from '../../hermes-client.service';
import { GroupDropdownComponent } from '../../groups/group-dropdown/group-dropdown.component';
import { Group } from '../../shared/models/group';
import { Policy } from '../../shared/models/policy';
import { PolicyAddFormComponent } from '../policy-add-form/policy-add-form.component';

@Component({
  selector: 'policy-item-edit',
  imports: [
    GroupDropdownComponent,
    MatButtonModule,
    MatCardModule,
    MatFormFieldModule,
    MatIconModule,
    MatInputModule,
    ReactiveFormsModule,
    PolicyAddFormComponent
  ],
  templateUrl: './policy-item-edit.component.html',
  styleUrl: './policy-item-edit.component.scss'
})
export class PolicyItemEditComponent implements OnInit {
  private readonly client = inject(HermesClientService);
  readonly data = inject(MAT_DIALOG_DATA);
  readonly dialogRef = inject(MatDialogRef<PolicyItemEditComponent>);

  readonly groupControl = new FormControl<Group | string | undefined>(undefined, [Validators.required]);
  readonly pathControl = new FormControl('', [Validators.required]);
  readonly usageControl = new FormControl(1, [Validators.required, Validators.min(1), Validators.max(99)]);
  readonly spanControl = new FormControl(5000, [Validators.required, Validators.min(1000), Validators.max(86400)]);

  readonly groupErrorMessages = {
    'required': 'This field is required.'
  };

  readonly formGroup = new FormGroup({
    group: this.groupControl,
    path: this.pathControl,
    usage: this.usageControl,
    span: this.spanControl,
  });

  isNew: boolean = false;
  waitForResponse: boolean = false;

  ngOnInit(): void {
    this.isNew = this.data.isNew;

    if (!this.data.policy_id)
      return;

    const policy = this.data.policies.find((p: Policy) => p.id == this.data.policy_id);
    if (!policy)
      return;

    this.groupControl.setValue(policy.group_id);
    this.pathControl.setValue(policy.path);
    this.usageControl.setValue(policy.usage);
    this.spanControl.setValue(policy.span);
  }

  save() {
    if (this.formGroup.invalid || this.waitForResponse)
      return;

    this.waitForResponse = true;
    const group_id = (this.groupControl.value as Group)!.id;
    const path = this.pathControl.value!;
    const usage = this.usageControl.value!;
    const span = this.spanControl.value!;

    if (this.isNew) {
      this.client.first((d: any) => d.op == 4 && d.d.request.type == 'create_policy' && d.d.data.group_id == group_id && d.d.data.path == path && d.d.data.usage == usage && d.d.data.span == span)
        .subscribe({
          next: (d) => this.dialogRef.close(d.d.data),
          error: () => this.waitForResponse = false,
          complete: () => this.waitForResponse = false,
        });
      this.client.createPolicy(group_id, path, usage, span);
    } else {
      this.client.first((d: any) => d.op == 4 && d.d.request.type == 'update_policy' && d.d.data.id == this.data.policy_id && d.d.data.group_id == group_id && d.d.data.path == path && d.d.data.usage == usage && d.d.data.span == span)
        .subscribe({
          next: (d) => this.dialogRef.close(d.d.data),
          error: () => this.waitForResponse = false,
          complete: () => this.waitForResponse = false,
        });
      this.client.updatePolicy(this.data.policy_id, group_id, path, usage, span);
    }
  }
}
